@@ -9,7 +9,7 @@
 #
 package Config::Model::Tk::LeafEditor ;
 {
-  $Config::Model::Tk::LeafEditor::VERSION = '1.331';
+  $Config::Model::Tk::LeafEditor::VERSION = '1.337';
 }
 
 use strict;
@@ -164,15 +164,27 @@ sub cleanup {
 sub add_buttons {
     my ($cw,$frame) = @_ ;
     my $bframe = $frame->Frame->pack() ;
-    $bframe -> Button ( -text => 'Reset',
-                        -command => sub { $cw->reset_value ; },
-                      ) -> pack(-side => 'left') ;
-    $bframe -> Button ( -text => 'Delete',
+
+    my $balloon = $cw->Balloon(-state => 'balloon') ;
+    
+    my $reset_b = $bframe -> Button ( 
+        -text => 'Reset',
+        -command => sub { $cw->reset_value ; },
+    ) -> pack(-side => 'left') ;
+    $balloon->attach($reset_b, -msg => "reset entry value from tree value") ;
+    
+    my $del_label = defined $cw->{leaf}->fetch_standard ? 'Back to default' : 'Delete' ;
+    $bframe -> Button ( -text => $del_label,
                         -command => sub { $cw->delete},
                       ) -> pack(-side => 'left') ;
-    $bframe -> Button ( -text => 'Store',
-                        -command => sub { $cw->store},
-                      ) -> pack(-side => 'right') ;
+    my $store_b = $bframe -> Button (
+        -text => 'Store',
+        -command => sub { $cw->store},
+    ) -> pack(-side => 'right') ;
+    
+    $balloon->attach($store_b, -msg => "store entry value in config tree") ;
+    
+    
     return $bframe ;
 }
 
@@ -228,7 +240,7 @@ sub delete {
         # trigger redraw of Tk Tree
         $cw->reset_value ;
         $cw->update_warning($cw->{leaf}) ;
-        $cw->parent->parent->parent->parent->reload(1) ;
+        $cw->parent->parent->parent->parent->reload() ;
     }
 }
 
@@ -250,7 +262,7 @@ sub store {
     }
     else {
         # trigger redraw of Tk Tree
-        $cw->{store_cb}->($cw->{leaf}) ;
+        $cw->{store_cb}->() ;
         $cw->update_warning($cw->{leaf}) ;
     }
 }
@@ -290,7 +302,7 @@ sub exec_external_editor {
     my $fh = File::Temp->new ; ;
     die "Can't open temp file:$!" unless defined $fh ;
     binmode($fh,":utf8");
-    $fh->print($cw->{value});
+    $fh->print($cw->{e_widget}->get('1.0','end'));
     $fh->close ;
 
     # See mastering Perl/Tk p382
@@ -309,8 +321,15 @@ sub exec_external_editor {
     }
     $h->autoflush(1);
     $cw->fileevent($h, 'readable' => [\&_read_stdout, $cw]);
+    
+    # prevent navigation in the tree (and destruction of this widget
+    # while the external editor is active). See mastering Perl/Tk p302
+    $cw->grab ;
 
     $cw->waitVariable(\$cw->{ed_done});
+    
+    $cw->grabRelease; 
+    
     my $new_v = read_file($fh->filename) ;
     print "exec_external_editor done with '$new_v'\n";
     $cw->store($new_v);
