@@ -11,9 +11,10 @@
 
 package Config::Model::TkUI ;
 {
-  $Config::Model::TkUI::VERSION = '1.331';
+  $Config::Model::TkUI::VERSION = '1.337';
 }
 
+use 5.10.1 ;
 use strict;
 use warnings ;
 use Carp ;
@@ -119,14 +120,13 @@ sub Populate {
     my $extra_menu = delete $args->{'-extra-menu'} || [] ;
 
     my $title = delete $args->{'-title'}
-              || "config-edit ".$cw->{root}->config_class_name ;
+              || $0." ".$cw->{root}->config_class_name ;
 
     # check unknown parameters
     croak "Unknown parameter ",join(' ',keys %$args) if %$args;
 
     # initialize internal attributes
     $cw->{location} = '';
-    $cw->{modified_data} = 0;
 
     $cw->setup_scanner() ;
 
@@ -140,6 +140,7 @@ sub Populate {
 		      [ qw/command reload -command/, sub{ $cw->reload }],
 		      [ command => 'check for errors',    -command => sub{ $cw->check(1)} ],
 		      [ command => 'check for warnings',  -command => sub{ $cw->check(1,1)} ],
+		      [ command => 'show unsaved changes',  -command => sub{ $cw->show_changes;} ],
 		      [ qw/command save   -command/, sub{ $cw->save }],
 		      [ command => 'save in dir ...',
                         -command => sub{ $cw->save_in_dir ;} ],
@@ -182,6 +183,14 @@ sub Populate {
     $loc_frame->Label(-text => 'location :') -> pack ( -side => 'left');
     $loc_frame->Label(-textvariable => \$cw->{location}) -> pack ( -side => 'left');
 
+    # create 'show only custom values'
+    $cw->{show_only_custom} = 0;
+    $loc_frame->Checkbutton(
+        -variable => \$cw->{show_only_custom},
+        -command => sub {$cw->reload} ,
+    ) -> pack (-side => 'right') ;
+    $loc_frame->Label(-text => 'show only custom values') ->pack(-side => 'right') ;
+
     # add bottom frame
     my $bottom_frame = $cw->Frame
       ->pack  (qw/-pady 0 -fill both -expand 1/ ) ;
@@ -293,10 +302,13 @@ my $pom = $parser->parse_file(__FILE__)
 
 my $help_text;
 my $todo_text;
+my $info_text;
 foreach my $head1 ($pom->head1()) {
     $help_text = Pod::POM::View::Text->view_head1($head1)
-      if $head1->title eq 'USAGE';
-    $todo_text = $head1->content if $head1->title eq 'TODO';
+        if $head1->title eq 'USAGE';
+    $info_text = Pod::POM::View::Text->view_head1($head1)
+        if $head1->title =~ /more information/i ;
+
 }
 
 sub add_help_menu {
@@ -305,15 +317,15 @@ sub add_help_menu {
     my $about_sub = sub {
 	$cw->Dialog(-title => 'About',
 		    -text => "Config::Model::TkUI \n"
-		    ."(c) 2008-2011 Dominique Dumont \n"
+		    ."(c) 2008-2012 Dominique Dumont \n"
 		    ."Licensed under LGPLv2\n"
 		   ) -> Show ;
     };
 
-    my $todo_sub = sub {
+    my $info_sub = sub {
 	my $db = $cw->DialogBox( -title => 'TODO');
-	my $text = $db -> add('ROText')->pack ;
-	$text ->insert('end',$todo_text) ;
+	my $text = $db -> add('Scrolled','ROText')->pack ;
+	$text ->insert('end',$info_text) ;
 	$db-> Show ;
     };
 
@@ -335,8 +347,8 @@ sub add_help_menu {
     };
 
     my $help_items = [[ qw/command About -command/, $about_sub ],
-		      [ qw/command Todo  -command/, $todo_sub  ],
 		      [ qw/command Usage -command/, $help_sub  ],
+		      [ command => 'More info', -command => $info_sub  ],
 		      [ command => "$class help", -command => $man_sub ],
 		     ] ;
     $menubar->cascade( -label => 'Help', -menuitems => $help_items ) ;
@@ -418,27 +430,34 @@ sub save {
             -> Show ;
        }
     }
-    $cw->{modified_data} = 0 ;
 }
 
 sub save_if_yes {
     my $cw =shift ;
     my $text = shift || "Save data ?" ;
 
-    if ($cw->{modified_data}) {
+    my $ok_to_quit = 1 ;
+    if ($cw->{root}->instance->needs_save) {
 	my $answer = $cw->Dialog(-title => "quit",
 				 -text  => $text,
-				 -buttons => [ qw/yes no/],
+				 -buttons => [ qw/yes no cancel/, 'show changes'],
 				 -default_button => 'yes',
 				)->Show ;
-	$cw->save if $answer eq 'yes';
+	given ($answer) {
+            when ('yes') { $cw->save; $ok_to_quit = 1 ;}
+            when ('no')  {            $ok_to_quit = 1 ;}
+            when ('cancel')  {            $ok_to_quit = 0 ;}
+            when (/show/)  { $cw->show_changes(sub{$cw -> save_if_yes}) ;  $ok_to_quit = 0 ;}
+	}
     }
+
+    return $ok_to_quit ;
 }
 
 sub quit {
     my $cw = shift ;
 
-    $cw->save_if_yes ;
+    $cw->save_if_yes || return ;
 
     if (defined $cw->{quit} and $cw->{quit} eq 'soft') {
 	$cw->destroy ;
@@ -449,10 +468,25 @@ sub quit {
     }
 }
 
+sub show_changes {
+    my $cw = shift ;
+    my $cb = shift ;
+    
+    my $changes = $cw->{root}->instance->list_changes ;
+    my $change_widget = $cw->Toplevel ;
+    $change_widget
+        -> Scrolled('ROText')
+        -> pack(-expand => 1, -fill => 'both')
+        -> insert('1.0',$changes);
+    $change_widget->Button(
+        -command => sub {$change_widget->destroy; $cb ->() if defined $cb;} ,
+        -text => 'ok',
+    )->pack ;
+}
+
 sub reload {
     my $cw =shift ;
-    carp "reload: too many parameters" if @_ > 2 ;
-    my $is_modif           = shift || 0; # whether values where modified
+    carp "reload: too many parameters" if @_ > 1 ;
     my $force_display_path = shift ;     # force open editor on this path
 
     $logger->trace("reloading tk tree".
@@ -460,7 +494,6 @@ sub reload {
 		  ) ;
 
     my $tree = $cw->{tktree} ;
-    $cw->{modified_data} = 1 if $is_modif ;
 
     my $instance_name = $cw->{root}->instance->name ;
 
@@ -549,7 +582,7 @@ sub force_element_display {
     my $elt_obj = shift ;
 
     $logger->trace( "force display of ".$elt_obj->location );
-    $cw->reload(0, $elt_obj->location) ;
+    $cw->reload($elt_obj->location) ;
 }
 
 sub prune {
@@ -802,10 +835,11 @@ sub disp_leaf {
     my $value = $leaf_object->fetch(check => 'no', silent => 1) ;
     my $tkt = $cw->{tktree} ;
 
-    my $img ;
+    my ($is_customised, $img) ;
     {
 	no warnings qw/uninitialized/ ;
-	$img = $cust_img if (defined $value and $std_v ne $value) ;
+        $is_customised = ! ( $std_v eq $value)  ;
+	$img = $cust_img if $is_customised ;
 	$img = $warn_img if $leaf_object->warning_msg ;
 	$img = $error_img if $leaf_object->error_msg;
     }
@@ -824,6 +858,9 @@ sub disp_leaf {
     $tkt->itemCreate($path,2, -text => $cw->trim_value($value)) ;
 
     $tkt->itemCreate($path,3, -text => $cw->trim_value($std_v)) ;
+    
+    my $meth = ($cw->{show_only_custom} and not $is_customised) ? 'hide' : 'show' ;
+    $tkt->$meth(entry => $path) ; 
 }
 
 sub disp_node {
@@ -1027,7 +1064,7 @@ sub edit_paste {
 	}
     }
 
-    $cw->reload(1) if @$cut_buf;
+    $cw->reload() if @$cut_buf;
 }
 
 sub wizard {
@@ -1051,11 +1088,17 @@ sub setup_wizard {
     return $cw->ConfigModelWizard
       (
 	-root     => $cw->{root},
-	-store_cb => sub{ $cw->{modified_data} = 1 ;},
 	-end_cb   => $end_sub,
       ) ;
 }
 
+# FIXME: need to be able to search different types.
+# 2 choices
+# - destroy and re-create the searcher when it's modified
+# - change the searcher (TreeSearcher) to accept type modif
+# For the latter: it would be better to accept a set of types instead of
+# all or just one type (to implement a set of check buttons)
+
 sub create_find_widget {
     my $cw = shift ;
     my $f = $cw -> Frame(-relief => 'ridge', -borderwidth => 1 , ) ;
@@ -1092,6 +1135,12 @@ sub create_find_widget {
         ) -> pack(-side => 'left');
     }
     
+    # bind Return (or Enter) key 
+    $e->bind( 
+        '<Key-Return>',
+        sub { $cw -> find_item('next', $searcher,\$search, \@result) ;}
+    ) ;
+    
     return $f ;
 }
 
@@ -1191,6 +1240,12 @@ configuration tree. Beware, there's no "undo" operation.
 
 =item *
 
+Before saving your modifications, you can review the change list with the 
+menu entry C<< File -> show unsaved changes >>. This list is cleared after 
+performing a C<< File -> save >>.
+
+=item *
+
 Pasting cut buffer into:
 
 =over
@@ -1243,7 +1298,6 @@ items (mostly missing mandatory values).
 
 =head1 TODO
 
-- Document widget options. (-root_model and -store_sub, -quit)
 - add tabular view ?
 - expand the whole tree at once
 - add plug-in mechanism so that dedicated widget
@@ -1254,9 +1308,27 @@ items (mostly missing mandatory values).
 
 Dominique Dumont, (ddumont at cpan dot org)
 
+=head1 More information
+
+=over
+
+=item *
+
+See L<Config::Model home page|https://github.com/dod38fr/config-model/wiki>
+
+=item *
+
+Or L<Author's blog|http://ddumont.wordpress.com> where you can find many post about L<Config::Model>.
+
+=item *
+
+Send a mail to Config::Model user mailing list: config-model-users at lists.sourceforge.net
+
+=back
+
 =head1 LICENSE
 
-    Copyright (c) 2008-2011 Dominique Dumont.
+    Copyright (c) 2008-2012 Dominique Dumont.
 
     This file is part of Config-Model.
 
